@@ -18,9 +18,10 @@
 // *************************************************************************
 
 #![recursion_limit = "128"]
-#![deny(
+#![warn(
+  bad_style,
   dead_code,
-  duplicate_associated_type_bindings,
+  future_incompatible,
   illegal_floating_point_literal_pattern,
   improper_ctypes,
   intra_doc_link_resolution_failure,
@@ -29,12 +30,16 @@
   missing_debug_implementations,
   no_mangle_generic_items,
   non_shorthand_field_patterns,
+  nonstandard_style,
   overflowing_literals,
   path_statements,
   patterns_in_fns_without_body,
   plugin_as_library,
   private_in_public,
   proc_macro_derive_resolution_fallback,
+  renamed_and_removed_lints,
+  rust_2018_compatibility,
+  rust_2018_idioms,
   safe_packed_borrows,
   stable_features,
   trivial_bounds,
@@ -56,14 +61,6 @@
   where_clauses_object_safety,
   while_true
 )]
-#![warn(
-  bad_style,
-  future_incompatible,
-  nonstandard_style,
-  renamed_and_removed_lints,
-  rust_2018_compatibility,
-  rust_2018_idioms
-)]
 
 //! A crate providing supporting testing infrastructure for the
 //! `nitrokey` crate and its users.
@@ -87,7 +84,6 @@
 //! Right now we make a few simplifying assumptions that, although not
 //! changing what can be expressed and tested, can lead to unexpected
 //! error messages when not known:
-//! - the parameter naming the device to test on has to be `device`
 //! - the parameter has to be an owned object, not a reference
 //! - parameter types are pattern matched against "Storage", "Pro", and
 //!   "DeviceWrapper"; that means `use ... as` declarations will not work
@@ -95,8 +91,6 @@
 
 extern crate proc_macro;
 
-use std::sync::atomic;
-
 use proc_macro::TokenStream;
 use proc_macro2::Ident;
 use proc_macro2::Literal;
@@ -122,11 +116,20 @@ const NITROKEY_GROUP_PRO: &str = "pro";
 const NITROKEY_GROUP_STORAGE: &str = "storage";
 
 
+/// The kind of argument a test function accepts.
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
+enum ArgumentType {
+  /// Pass an actual device handle to the test function.
+  Device,
+  /// Pass in a device wrapper to the test function.
+  DeviceWrapper,
+  /// Pass a `Model` object to the test function.
+  Model,
+}
+
 /// A type used to determine what Nitrokey device to test on.
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
 enum SupportedDevice {
-  /// No device must be present.
-  None,
   /// Only the Nitrokey Pro is supported.
   Pro,
   /// Only the Nitrokey Storage is supported.
@@ -135,22 +138,76 @@ enum SupportedDevice {
   Any,
 }
 
-
-/// A type defining what kind of Nitrokey device to connect to and
-/// whether to wrap it.
-#[derive(Clone, Copy, Debug)]
-enum EmittedDevice {
-  /// Bail out if the connection succeeds, i.e., only run when no device
-  /// is present.
-  None,
-  /// Connect to and pass in a `nitrokey::Pro`.
+/// A type used for "filtering" what device types to emit test code for.
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
+enum Filter {
+  /// Only emit tests for a Nitrokey Pro.
   Pro,
-  /// Connect to and pass in a `nitrokey::Storage`.
+  /// Only emit tests for a Nitrokey Storage.
   Storage,
-  /// Connect to a `nitrokey::Pro` but pass it in as a `nitrokey::DeviceWrapper`.
-  WrappedPro,
-  /// Connect to a `nitrokey::Storage` but pass it in as a `nitrokey::DeviceWrapper`.
-  WrappedStorage,
+}
+
+impl Filter {
+  pub fn from_attribute(attr: &TokenStream) -> Option<Self> {
+    match attr.to_string().as_ref() {
+      "pro" => Some(Filter::Pro),
+      "storage" => Some(Filter::Storage),
+      "" => None,
+      _ => panic!("unexpected filter argument: {}", attr),
+    }
+  }
+}
+
+
+/// Apply a filter to a `SupportedDevice`.
+///
+/// Filtering basically produces the following outcomes:
+/// #[test]          fn foo();                      -> no device
+/// #[test(pro)]     fn foo();                      -> pro
+/// #[test(storage)] fn foo();                      -> storage
+/// #[test]          fn foo(device: Pro);           -> pro
+/// #[test(pro)]     fn foo(device: Pro);           -> pro
+/// #[test(storage)] fn foo(device: Pro);           -> error
+/// #[test]          fn foo(device: Storage);       -> storage
+/// #[test(pro)]     fn foo(device: Storage);       -> error
+/// #[test(storage)] fn foo(device: Storage);       -> storage
+/// #[test]          fn foo(device: DeviceWrapper); -> any
+/// #[test(pro)]     fn foo(device: DeviceWrapper); -> pro
+/// #[test(storage)] fn foo(device: DeviceWrapper); -> storage
+/// #[test]          fn foo(model: Model);          -> any
+/// #[test(pro)]     fn foo(model: Model);          -> pro
+/// #[test(storage)] fn foo(model: Model);          -> storage
+fn filter_device(
+  device: Option<SupportedDevice>,
+  filter: Option<Filter>,
+) -> Option<SupportedDevice>
+{
+  match device {
+    None => match filter {
+      None => None,
+      // As can be seen from the table above, we have some logic in here
+      // (for the user's convenience) that is no longer strictly a
+      // filter, but rather an addition. That is done mostly for the
+      // user's convenience.
+      Some(Filter::Pro) => Some(SupportedDevice::Pro),
+      Some(Filter::Storage) => Some(SupportedDevice::Storage),
+    },
+    Some(SupportedDevice::Pro) => match filter {
+      None |
+      Some(Filter::Pro) => Some(SupportedDevice::Pro),
+      Some(Filter::Storage) => panic!("unable to combine 'storage' filter with Pro device"),
+    },
+    Some(SupportedDevice::Storage) => match filter {
+      None |
+      Some(Filter::Storage) => Some(SupportedDevice::Storage),
+      Some(Filter::Pro) => panic!("unable to combine 'pro' filter with Storage device"),
+    },
+    Some(SupportedDevice::Any) => match filter {
+      None => Some(SupportedDevice::Any),
+      Some(Filter::Pro) => Some(SupportedDevice::Pro),
+      Some(Filter::Storage) => Some(SupportedDevice::Storage),
+    },
+  }
 }
 
 
@@ -176,6 +233,19 @@ impl AsRef<str> for DeviceGroup {
   }
 }
 
+impl From<Option<SupportedDevice>> for DeviceGroup {
+  fn from(device: Option<SupportedDevice>) -> Self {
+    match device {
+      None => DeviceGroup::No,
+      Some(device) => match device {
+        SupportedDevice::Pro => DeviceGroup::Pro,
+        SupportedDevice::Storage => DeviceGroup::Storage,
+        SupportedDevice::Any => panic!("an Any device cannot belong to a group"),
+      }
+    }
+  }
+}
+
 impl quote::ToTokens for DeviceGroup {
   fn to_tokens(&self, tokens: &mut Tokens) {
     tokens.append(Literal::string(self.as_ref()))
@@ -210,6 +280,15 @@ impl quote::ToTokens for DeviceGroup {
 /// }
 /// ```
 ///
+/// Test functionality on any Nitrokey device, but leave the device
+/// connection to the user and just provide the model:
+/// ```rust,no_run
+/// #[nitrokey_test::test]
+/// fn some_other_nitrokey_test(model: nitrokey::Model) {
+///   // Connect to a device of the provided model.
+/// }
+/// ```
+///
 /// Test functionality on a Nitrokey Pro device:
 /// ```rust,no_run
 /// #[nitrokey_test::test]
@@ -218,6 +297,23 @@ impl quote::ToTokens for DeviceGroup {
 /// }
 /// ```
 ///
+/// Test functionality on a Nitrokey Pro device, but leave the device
+/// connection to the user:
+/// ```rust,no_run
+/// #[nitrokey_test::test(pro)]
+/// fn some_other_pro_test() {
+///   // Do something on a Pro device.
+/// }
+/// ```
+///
+/// A model can be provided optionally, like so:
+/// ```rust,no_run
+/// #[nitrokey_test::test(pro)]
+/// fn some_other_pro_test(model: nitrokey::Model) {
+///   assert_eq!(model, nitrokey::Model::Pro);
+/// }
+/// ```
+///
 /// Test functionality on a Nitrokey Storage device:
 /// ```rust,no_run
 /// #[nitrokey_test::test]
@@ -235,36 +331,39 @@ impl quote::ToTokens for DeviceGroup {
 /// ```
 #[proc_macro_attribute]
 pub fn test(attr: TokenStream, item: TokenStream) -> TokenStream {
-  // Bail out if user tried to pass additional arguments. E.g.,
-  // #[nitrokey_test::test(foo = "bar")
-  if !attr.is_empty() {
-    panic!("unsupported attributes supplied: {}", attr);
-  }
+  let input = syn::parse_macro_input!(item as syn::ItemFn);
+  let filter = Filter::from_attribute(&attr);
+  let dev_type = determine_device(&input.sig.inputs);
+  let (device, argument) = dev_type
+    .map_or((None, None), |(device, argument)| {
+      (Some(device), Some(argument))
+    });
+  let device = filter_device(device, filter);
+
   // Make clippy happy.
   drop(attr);
 
-  let input = syn::parse_macro_input!(item as syn::ItemFn);
-  let dev_type = determine_device(&input.decl.inputs);
+  match device {
+    None => {
+      let name = format!("{}", &input.sig.ident);
+      expand_wrapper(name, None, argument, &input)
+    },
+    Some(SupportedDevice::Pro) => {
+      let name = format!("{}", &input.sig.ident);
+      expand_wrapper(name, device, argument, &input)
+    },
+    Some(SupportedDevice::Storage) => {
+      let name = format!("{}", &input.sig.ident);
+      expand_wrapper(name, device, argument, &input)
+    },
+    Some(SupportedDevice::Any) => {
+      let name = format!("{}_pro", &input.sig.ident);
+      let dev = Some(SupportedDevice::Pro);
+      let pro = expand_wrapper(name, dev, argument, &input);
 
-  match dev_type {
-    SupportedDevice::None => {
-      let name = format!("{}", &input.ident);
-      expand_wrapper(name, EmittedDevice::None, &input)
-    },
-    SupportedDevice::Pro => {
-      let name = format!("{}", &input.ident);
-      expand_wrapper(name, EmittedDevice::Pro, &input)
-    },
-    SupportedDevice::Storage => {
-      let name = format!("{}", &input.ident);
-      expand_wrapper(name, EmittedDevice::Storage, &input)
-    },
-    SupportedDevice::Any => {
-      let name = format!("{}_pro", &input.ident);
-      let pro = expand_wrapper(name, EmittedDevice::WrappedPro, &input);
-
-      let name = format!("{}_storage", &input.ident);
-      let storage = expand_wrapper(name, EmittedDevice::WrappedStorage, &input);
+      let name = format!("{}_storage", &input.sig.ident);
+      let dev = Some(SupportedDevice::Storage);
+      let storage = expand_wrapper(name, dev, argument, &input);
 
       // Emit a test for both the Pro and the Storage device.
       quote! {
@@ -276,35 +375,6 @@ pub fn test(attr: TokenStream, item: TokenStream) -> TokenStream {
   .into()
 }
 
-/// Emit code for a function, __nitrokey_mutex, that can be used to
-/// serialize accesses to the nitrokey by grabbing a mutex. This
-/// function is meant to be used by the test code that is generated by
-/// the attribute macro.
-/// Note that this code is only emitted on the very first invocation of
-/// this function, to ensure we only have one such function in the test
-/// module.
-fn expand_serializer() -> Tokens {
-  static EMITTED: atomic::AtomicBool = atomic::AtomicBool::new(false);
-
-  if !EMITTED.fetch_or(true, atomic::Ordering::Relaxed) {
-    quote! {
-      fn __nitrokey_mutex() -> &'static ::std::sync::Mutex<()> {
-        static mut MUTEX: Option<::std::sync::Mutex<()>> = None;
-        static ONCE: ::std::sync::Once = ::std::sync::Once::new();
-
-        ONCE.call_once(|| unsafe { MUTEX = Some(::std::sync::Mutex::new(())) });
-
-        match unsafe { &MUTEX } {
-          Some(mutex) => mutex,
-          None => unreachable!(),
-        }
-      }
-    }
-  } else {
-    quote! {}
-  }
-}
-
 fn expand_connect(group: DeviceGroup, ret_type: &syn::ReturnType) -> Tokens {
   let (ret, check) = match ret_type {
     syn::ReturnType::Default => (quote! { return }, quote! {.unwrap()}),
@@ -317,9 +387,9 @@ fn expand_connect(group: DeviceGroup, ret_type: &syn::ReturnType) -> Tokens {
   };
 
   let connect = match group {
-    DeviceGroup::No => quote! { ::nitrokey::connect() },
-    DeviceGroup::Pro => quote! { ::nitrokey::Pro::connect() },
-    DeviceGroup::Storage => quote! { ::nitrokey::Storage::connect() },
+    DeviceGroup::No => quote! { manager.connect() },
+    DeviceGroup::Pro => quote! { manager.connect_pro() },
+    DeviceGroup::Storage => quote! { manager.connect_storage() },
   };
 
   let connect_cond = if let DeviceGroup::No = group {
@@ -399,8 +469,121 @@ fn expand_connect(group: DeviceGroup, ret_type: &syn::ReturnType) -> Tokens {
   }
 }
 
+fn expand_arg<P>(
+  device: Option<SupportedDevice>,
+  argument: Option<ArgumentType>,
+  args: &punctuated::Punctuated<syn::FnArg, P>,
+) -> Tokens
+where
+  P: quote::ToTokens,
+{
+  // Based on the device we want to emit a test function for we recreate
+  // the expected full path of the type. That is necessary because
+  // client code may have a "use" and may just contain a `Pro`, for
+  // example, while we really need to work with the absolute path.
+  let arg_type = match device {
+    None => quote! {},
+    Some(device) => match argument {
+      None => quote! {},
+      Some(ArgumentType::Device) => match device {
+        SupportedDevice::Pro => quote! { ::nitrokey::Pro },
+        SupportedDevice::Storage => quote! { ::nitrokey::Storage },
+        SupportedDevice::Any => unreachable!(),
+      },
+      Some(ArgumentType::DeviceWrapper) => quote! { ::nitrokey::DeviceWrapper },
+      Some(ArgumentType::Model) => quote! { ::nitrokey::Model },
+    },
+  };
+
+  match args.first() {
+    Some(arg) => match arg {
+      syn::FnArg::Typed(pat_type) => {
+        let arg = syn::FnArg::Typed(syn::PatType {
+          attrs: Vec::new(),
+          pat: pat_type.pat.clone(),
+          colon_token: pat_type.colon_token,
+          ty: Box::new(syn::Type::Path(syn::parse_quote! { #arg_type })),
+        });
+        quote! { #arg }
+      }
+      _ => panic!("unexpected test function argument"),
+    },
+    None => quote! {},
+  }
+}
+
+fn expand_call(
+  device: Option<SupportedDevice>,
+  argument: Option<ArgumentType>,
+  wrappee: &syn::ItemFn,
+) -> Tokens
+{
+  let test_name = &wrappee.sig.ident;
+  let group = DeviceGroup::from(device);
+  let connect = expand_connect(group, &wrappee.sig.output);
+
+  let call = match device {
+    None => quote! { #test_name() },
+    Some(device) => match argument {
+      None => quote! { #test_name() },
+      Some(ArgumentType::Device) => quote! { #test_name(device) },
+      Some(ArgumentType::DeviceWrapper) => match device {
+        SupportedDevice::Pro => {
+          quote! {
+            #test_name(::nitrokey::DeviceWrapper::Pro(device))
+          }
+        },
+        SupportedDevice::Storage => {
+          quote! {
+            #test_name(::nitrokey::DeviceWrapper::Storage(device))
+          }
+        },
+        SupportedDevice::Any => unreachable!(),
+      },
+      Some(ArgumentType::Model) => {
+        let model = match device {
+          SupportedDevice::Pro => quote! { ::nitrokey::Model::Pro },
+          SupportedDevice::Storage => quote! { ::nitrokey::Model::Storage },
+          SupportedDevice::Any => unreachable!(),
+        };
+        quote! { #test_name(#model) }
+      }
+    },
+  };
+
+  match argument {
+    None |
+    Some(ArgumentType::Model) => {
+      // Make sure that if no device is passed in the user is still
+      // allowed to use nitrokey::take successfully by not keeping a
+      // Manager object lying around. We just need it to check whether or
+      // not to skip the test.
+      quote! {
+        {
+          let mut manager = ::nitrokey::force_take().unwrap();
+          let _ = #connect;
+        }
+        #call
+      }
+    },
+    Some(ArgumentType::Device) |
+    Some(ArgumentType::DeviceWrapper) => {
+      quote! {
+        let mut manager = ::nitrokey::force_take().unwrap();
+        let device = #connect;
+        #call
+      }
+    }
+  }
+}
+
 /// Emit code for a wrapper function around a Nitrokey test function.
-fn expand_wrapper<S>(fn_name: S, device: EmittedDevice, wrappee: &syn::ItemFn) -> Tokens
+fn expand_wrapper<S>(
+  fn_name: S,
+  device: Option<SupportedDevice>,
+  argument: Option<ArgumentType>,
+  wrappee: &syn::ItemFn,
+) -> Tokens
 where
   S: AsRef<str>,
 {
@@ -409,68 +592,55 @@ where
   // cannot be interpolated with quote!{} for lack of quote::ToTokens
   // implementations.
   let name = Ident::new(fn_name.as_ref(), Span::call_site());
-  let decl = &wrappee.decl;
+  let attrs = &wrappee.attrs;
   let body = &wrappee.block;
-  let ret_type = match &decl.output {
+  let test_name = &wrappee.sig.ident;
+  let test_arg = expand_arg(device, argument, &wrappee.sig.inputs);
+  let test_call = expand_call(device, argument, wrappee);
+
+  let ret_type = match &wrappee.sig.output {
     syn::ReturnType::Default => quote! {()},
     syn::ReturnType::Type(_, type_) => quote! {#type_},
   };
 
-  let connect_pro = expand_connect(DeviceGroup::Pro, &decl.output);
-  let connect_storage = expand_connect(DeviceGroup::Storage, &decl.output);
-
-  let connect = match device {
-    EmittedDevice::None => expand_connect(DeviceGroup::No, &decl.output),
-    EmittedDevice::Pro => quote! { let device = #connect_pro },
-    EmittedDevice::Storage => quote! { let device = #connect_storage },
-    EmittedDevice::WrappedPro => {
-      quote! {
-        let device = ::nitrokey::DeviceWrapper::Pro(#connect_pro)
-      }
-    },
-    EmittedDevice::WrappedStorage => {
-      quote! {
-        let device = ::nitrokey::DeviceWrapper::Storage(#connect_storage)
-      }
-    },
-  };
-
-  let serializer = expand_serializer();
   quote! {
-    #serializer
-
     #[test]
+    #(#attrs)*
     fn #name() -> #ret_type {
+      fn #test_name(#test_arg) -> #ret_type {
+        #body
+      }
+
       // Note that mutexes (and other locks) come with a poisoning
       // mechanism that (by default) prevents an acquisition of a mutex
       // that was held while the thread holding it panic'ed. We don't
       // care about that protection. There are no real invariants that
       // our mutex is protecting, it just synchronizes accesses to the
       // nitrokey device. As such, just override the protection.
-      let _guard = __nitrokey_mutex()
+      let _guard = ::nitrokey_test_state::mutex()
         .lock()
         .map_err(|err| err.into_inner());
-      #connect;
-      #body
+      #test_call
     }
   }
 }
 
-fn determine_device_for_arg(arg: &syn::FnArg) -> SupportedDevice {
+fn determine_device_for_arg(arg: &syn::FnArg) -> (SupportedDevice, ArgumentType) {
   match arg {
-    syn::FnArg::Captured(arg) => {
-      let type_ = &arg.ty;
-      match type_ {
+    syn::FnArg::Typed(pat_type) => {
+      let type_ = &pat_type.ty;
+      match &**type_ {
         syn::Type::Path(path) => {
           if path.path.segments.is_empty() {
             panic!("invalid function argument type: {}", quote! {#path});
           }
 
-          let type_ = format!("{}", path.path.segments.last().unwrap().value().ident);
+          let type_ = format!("{}", path.path.segments.last().unwrap().ident);
           match type_.as_ref() {
-            "Storage" => SupportedDevice::Storage,
-            "Pro" => SupportedDevice::Pro,
-            "DeviceWrapper" => SupportedDevice::Any,
+            "Model" => (SupportedDevice::Any, ArgumentType::Model),
+            "Storage" => (SupportedDevice::Storage, ArgumentType::Device),
+            "Pro" => (SupportedDevice::Pro, ArgumentType::Device),
+            "DeviceWrapper" => (SupportedDevice::Any, ArgumentType::DeviceWrapper),
             _ => panic!("unsupported function argument type: {}", type_),
           }
         },
@@ -484,13 +654,15 @@ fn determine_device_for_arg(arg: &syn::FnArg) -> SupportedDevice {
 
 /// Determine the kind of Nitrokey device a test function support, based
 /// on the type of its only parameter.
-fn determine_device<P>(args: &punctuated::Punctuated<syn::FnArg, P>) -> SupportedDevice
+fn determine_device<P>(
+  args: &punctuated::Punctuated<syn::FnArg, P>,
+) -> Option<(SupportedDevice, ArgumentType)>
 where
   P: quote::ToTokens,
 {
   match args.len() {
-    0 => SupportedDevice::None,
-    1 => determine_device_for_arg(args.first().unwrap().value()),
+    0 => None,
+    1 => Some(determine_device_for_arg(&args[0])),
     _ => panic!("functions used as Nitrokey tests can only have zero or one argument"),
   }
 }
@@ -498,10 +670,10 @@ where
 
 #[cfg(test)]
 mod tests {
+  use super::ArgumentType;
   use super::determine_device;
   use super::SupportedDevice;
 
-  use quote::quote;
   use syn;
 
 
@@ -511,9 +683,9 @@ mod tests {
       #[nitrokey_test::test]
       fn test_none() {}
     };
-    let dev_type = determine_device(&input.decl.inputs);
+    let dev_type = determine_device(&input.sig.inputs);
 
-    assert_eq!(dev_type, SupportedDevice::None)
+    assert_eq!(dev_type, None);
   }
 
   #[test]
@@ -522,9 +694,9 @@ mod tests {
       #[nitrokey_test::test]
       fn test_pro(device: nitrokey::Pro) {}
     };
-    let dev_type = determine_device(&input.decl.inputs);
+    let dev_type = determine_device(&input.sig.inputs);
 
-    assert_eq!(dev_type, SupportedDevice::Pro)
+    assert_eq!(dev_type, Some((SupportedDevice::Pro, ArgumentType::Device)));
   }
 
   #[test]
@@ -533,9 +705,9 @@ mod tests {
       #[nitrokey_test::test]
       fn test_storage(device: nitrokey::Storage) {}
     };
-    let dev_type = determine_device(&input.decl.inputs);
+    let dev_type = determine_device(&input.sig.inputs);
 
-    assert_eq!(dev_type, SupportedDevice::Storage)
+    assert_eq!(dev_type, Some((SupportedDevice::Storage, ArgumentType::Device)));
   }
 
   #[test]
@@ -544,9 +716,9 @@ mod tests {
       #[nitrokey_test::test]
       fn test_any(device: nitrokey::DeviceWrapper) {}
     };
-    let dev_type = determine_device(&input.decl.inputs);
+    let dev_type = determine_device(&input.sig.inputs);
 
-    assert_eq!(dev_type, SupportedDevice::Any)
+    assert_eq!(dev_type, Some((SupportedDevice::Any, ArgumentType::DeviceWrapper)));
   }
 
   #[test]
@@ -556,7 +728,7 @@ mod tests {
       #[nitrokey_test::test]
       fn test_pro(device: nitrokey::Pro, _: i32) {}
     };
-    let _ = determine_device(&input.decl.inputs);
+    let _ = determine_device(&input.sig.inputs);
   }
 
   #[test]
@@ -566,7 +738,7 @@ mod tests {
       #[nitrokey_test::test]
       fn test_self(&self) {}
     };
-    let _ = determine_device(&input.decl.inputs);
+    let _ = determine_device(&input.sig.inputs);
   }
 
   #[test]
@@ -577,7 +749,7 @@ mod tests {
       #[nitrokey_test::test]
       fn test_any(device: &nitrokey::DeviceWrapper) {}
     };
-    let _ = determine_device(&input.decl.inputs);
+    let _ = determine_device(&input.sig.inputs);
   }
 
   #[test]
@@ -587,6 +759,6 @@ mod tests {
       #[nitrokey_test::test]
       fn test_foobarbaz(device: nitrokey::FooBarBaz) {}
     };
-    let _ = determine_device(&input.decl.inputs);
+    let _ = determine_device(&input.sig.inputs);
   }
 }
